<?php

namespace Drupal\social_wall\Plugin\SocialNetwork;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\social_wall\Plugin\SocialNetworkBase;
use Exception;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\GuzzleException;

/**
 * Class InstagramSocialNetwork.
 *
 * @package Drupal\social_wall\Plugin\SocialNetwork
 *
 * @SocialNetwork(
 *   id = "instagram_social_network",
 *   label = @Translation("Instagram social network")
 * )
 */
class InstagramSocialNetwork extends SocialNetworkBase implements ContainerFactoryPluginInterface {

  /**
   * HTTP Client.
   *
   * @var \GuzzleHttp\Client
   *   This is http client.
   */
  protected $httpClient;

  /**
   * Config Factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   *   This is config factory.
   */
  protected $configFactory;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    TranslationManager $translation_manager,
    CacheBackendInterface $cache_backend,
    LoggerChannelFactoryInterface $logger_factory,
    Client $http_client,
    ConfigFactoryInterface $config_factory) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $translation_manager, $cache_backend, $logger_factory);
    $this->httpClient = $http_client;
    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('string_translation'),
      $container->get('cache.default'),
      $container->get('logger.factory'),
      $container->get('http_client'),
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getLabel() {
    return 'Instagram';
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $third_party_settings = []) {
    $form = [];

    $max_length = 300;
    $form['token'] = [
      '#type' => 'textfield',
      '#title' => $this->translationManager->translate('Long-live token'),
      '#description' => $this->translationManager->translate('Max input : @max characters.', ['@max' => $max_length]),
      '#maxlength' => $max_length,
      '#default_value' => $third_party_settings['token'] ?? '',
      '#required' => TRUE,
    ];

    $form['nb_of_posts'] = [
      '#type' => 'select',
      '#title' => $this->translationManager->translate('Number of posts'),
      '#description' => $this->translationManager->translate('The number of posts to display.'),
      '#options' => array_combine(range(1, 10), range(1, 10)),
      '#default_value' => $third_party_settings['nb_of_posts'] ?? 1,
      '#required' => TRUE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function render() {
    $build = [];

    // If data has been cached, return cached data.
    $cached_results = $this->cacheBackend->get('social_wall_instagram_data' . $this->configuration['token']);
    if ($cached_results && ($cached_results->valid)) {
      return $cached_results->data;
    }

    $client = $this->httpClient;
    $token = $this->configuration['token'];
    $cached_token = $this->cacheBackend->get('social_wall_instagram_token' . $this->configuration['token']);

    /* If no cached token or cached token is expired, refresh
    token (long-lived tokens have a 2 months duration). */
    if ((!$cached_token || !$cached_token->valid) && $token) {
      try {
        $endpoint_url = "https://graph.instagram.com/refresh_access_token?grant_type=ig_refresh_token&access_token=$token";
        $request = $client->get($endpoint_url);
        $response = json_decode($request->getBody(), TRUE);

        /* Update Instagram token in configuration (be sure that
        the related config file is excluded from versioning). */
        $config = $this->configFactory->getEditable('social_wall.social_network_config.instagram');
        $tp_settings = $config->get('third_party_settings');
        $tp_settings['social_wall']['sn_config']['instagram']['token'] = $response['access_token'];
        $config->set('third_party_settings', $tp_settings);
        $config->save(TRUE);
        $token = $response['access_token'];

        // Cache token, to prevent excessive token refresh.
        $this->cacheBackend->set(
          'social_wall_instagram_token' . $this->configuration['token'],
          $response['access_token'],
          time() + self::getDataCacheTime()
        );
      }
      catch (Exception $e) {
        $this->loggerFactory->get('social_wall')->error('Instagram : @error', ['@error' => $e->getMessage()]);
      }
    }

    // Retrieve Instagram data.
    // @see https://developers.facebook.com/docs/instagram-basic-display-api/reference/media#fields.
    try {
      $options = [
        'headers' => [
          'Content-Type' => 'application/json',
          'Accept' => 'application/json',
          'Authorization' => "Bearer $token",
        ],
      ];
      $response = $client->get('https://graph.instagram.com/me/media?fields=id,caption,media_url,permalink,timestamp', $options);
      $posts = json_decode($response->getBody()->getContents(), TRUE)['data'] ?? [];

      $build = [
        '#theme' => 'social_network_instagram_block',
        '#elements' => [],
      ];

      $nb_of_posts = $this->configuration['nb_of_posts'] ?? 1;
      for ($i = 0; $i < $nb_of_posts; $i++) {
        if (empty($posts[$i])) {
          break;
        }

        $text = strlen($posts[$i]['caption']) < 145 ? $posts[$i]['caption'] : substr($posts[$i]['caption'], 0, 145) . '...';
        $build['#elements'][] = [
          'image_url' => $posts[$i]['media_url'],
          'creation_timestamp' => $posts[$i]['timestamp'],
          'body_text' => $text,
          'post_url' => $posts[$i]['permalink'],
        ];
      }
    }
    catch (GuzzleException $guzzleException) {
      $this->loggerFactory->get('social_wall')->error('Instagram : @error', ['@error' => $guzzleException->getMessage()]);
    }

    // Set block cache.
    $build['#cache']['max-age'] = self::getDataCacheTime();

    // Cache data.
    $this->cacheBackend->set('social_wall_instagram_data' . $this->configuration['token'], $build, time() + self::getDataCacheTime()
    );

    return $build;
  }

}
